
extern WMKey gMouseWinMove;
extern WMKey gMouseWinSize;
extern WMKey gMouseWinRaise;
extern WMKey gMouseWinLower;
extern WMKey gMouseWinTabbing;

extern WMKey gKeyWinRaise;
extern WMKey gKeyWinOccupyAll;
extern WMKey gKeyWinLower;
extern WMKey gKeyWinClose;
extern WMKey gKeyWinRestore;
extern WMKey gKeyWinPrev;
extern WMKey gKeyWinNext;
extern WMKey gKeyWinMove;
extern WMKey gKeyWinSize;
extern WMKey gKeyWinMinimize;
extern WMKey gKeyWinMaximize;
extern WMKey gKeyWinMaximizeVert;
extern WMKey gKeyWinMaximizeHoriz;
extern WMKey gKeyWinFullscreen;
extern WMKey gKeyWinHide;
extern WMKey gKeyWinRollup;
extern WMKey gKeyWinArrangeN;
extern WMKey gKeyWinArrangeNE;
extern WMKey gKeyWinArrangeE;
extern WMKey gKeyWinArrangeSE;
extern WMKey gKeyWinArrangeS;
extern WMKey gKeyWinArrangeSW;
extern WMKey gKeyWinArrangeW;
extern WMKey gKeyWinArrangeNW;
extern WMKey gKeyWinArrangeC;
extern WMKey gKeyWinTileLeft;
extern WMKey gKeyWinTileRight;
extern WMKey gKeyWinTileTop;
extern WMKey gKeyWinTileBottom;
extern WMKey gKeyWinTileTopLeft;
extern WMKey gKeyWinTileTopRight;
extern WMKey gKeyWinTileBottomLeft;
extern WMKey gKeyWinTileBottomRight;
extern WMKey gKeyWinTileCenter;
extern WMKey gKeyWinSmartPlace;
extern WMKey gKeyWinMenu;
extern WMKey gKeySysSwitchNext;
extern WMKey gKeySysSwitchLast;
extern WMKey gKeySysSwitchClass;
extern WMKey gKeySysWinNext;
extern WMKey gKeySysWinPrev;
extern WMKey gKeyTaskBarSwitchNext;
extern WMKey gKeyTaskBarSwitchPrev;
extern WMKey gKeyTaskBarMoveNext;
extern WMKey gKeyTaskBarMovePrev;
extern WMKey gKeySysWinMenu;
extern WMKey gKeySysDialog;
extern WMKey gKeySysMenu;
extern WMKey gKeySysWindowList;
extern WMKey gKeySysWinListMenu;
extern WMKey gKeySysKeyboardNext;
extern WMKey gKeySysAddressBar;
extern WMKey gKeySysWorkspacePrev;
extern WMKey gKeySysWorkspaceNext;
extern WMKey gKeySysWorkspaceLast;
extern WMKey gKeySysWorkspacePrevTakeWin;
extern WMKey gKeySysWorkspaceNextTakeWin;
extern WMKey gKeySysWorkspaceLastTakeWin;
extern WMKey gKeySysWorkspace1;
extern WMKey gKeySysWorkspace2;
extern WMKey gKeySysWorkspace3;
extern WMKey gKeySysWorkspace4;
extern WMKey gKeySysWorkspace5;
extern WMKey gKeySysWorkspace6;
extern WMKey gKeySysWorkspace7;
extern WMKey gKeySysWorkspace8;
extern WMKey gKeySysWorkspace9;
extern WMKey gKeySysWorkspace10;
extern WMKey gKeySysWorkspace11;
extern WMKey gKeySysWorkspace12;
extern WMKey gKeySysWorkspace1TakeWin;
extern WMKey gKeySysWorkspace2TakeWin;
extern WMKey gKeySysWorkspace3TakeWin;
extern WMKey gKeySysWorkspace4TakeWin;
extern WMKey gKeySysWorkspace5TakeWin;
extern WMKey gKeySysWorkspace6TakeWin;
extern WMKey gKeySysWorkspace7TakeWin;
extern WMKey gKeySysWorkspace8TakeWin;
extern WMKey gKeySysWorkspace9TakeWin;
extern WMKey gKeySysWorkspace10TakeWin;
extern WMKey gKeySysWorkspace11TakeWin;
extern WMKey gKeySysWorkspace12TakeWin;
extern WMKey gKeySysTileVertical;
extern WMKey gKeySysTileHorizontal;
extern WMKey gKeySysCascade;
extern WMKey gKeySysArrange;
extern WMKey gKeySysArrangeIcons;
extern WMKey gKeySysMinimizeAll;
extern WMKey gKeySysHideAll;
extern WMKey gKeySysUndoArrange;
extern WMKey gKeySysShowDesktop;
extern WMKey gKeySysCollapseTaskBar;

// vim: set sw=4 ts=4 et:
